"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const util = require("util");
const chalk_1 = require("chalk");
class CloudFormationResourceCounterPlugin {
    constructor(serverless, options) {
        this.serverless = serverless;
        this.options = options;
        this.hooks = {
            'after:deploy:deploy': this.process.bind(this),
        };
        this.warningThresholdOption = this.serverless.service.custom.warningThreshold;
    }
    get stackName() {
        const stackName = this.serverless.getProvider('aws').naming.stackName;
        return stackName || util.format('%s-%s', this.serverless.service.getServiceName(), this.serverless.getProvider('aws').getStage());
    }
    fetch(request) {
        return this.serverless.getProvider('aws').request('CloudFormation', 'listStackResources', request, this.serverless.getProvider('aws').getStage(), this.serverless.getProvider('aws').getRegion());
    }
    fetchStackResources() {
        return __awaiter(this, void 0, void 0, function* () {
            const stackResources = [];
            let result = yield this.fetch({ StackName: this.stackName });
            result.StackResourceSummaries.forEach((stackItem) => {
                stackResources.push(stackItem);
            });
            let morePages = result.NextToken ? true : false;
            while (morePages) {
                const request = {
                    NextToken: result.NextToken,
                    StackName: this.stackName,
                };
                result = yield this.fetch(request);
                result.StackResourceSummaries.forEach((stackItem) => {
                    stackResources.push(stackItem);
                });
                morePages = result.NextToken ? true : false;
            }
            return Promise.all(stackResources);
        });
    }
    count(resources) {
        return resources.length;
    }
    process() {
        Promise.resolve()
            .then(() => this.fetchStackResources())
            .then((response) => {
            const resourceCount = this.count(response);
            let message = util.format('CloudFormation resource count: %d', resourceCount);
            if (this.warningThresholdOption && resourceCount >= this.warningThresholdOption) {
                message += `\n${chalk_1.default.red('WARNING:')}\n`;
                message += `${chalk_1.default.red('AWS CloudFormation has a hard limit of 200 resources for a deployed stack!')}\n`;
            }
            this.serverless.cli.log(message);
        })
            .catch((error) => this.serverless.cli.log(util.format('Cannot count: %s!', error.message)));
    }
}
exports.CloudFormationResourceCounterPlugin = CloudFormationResourceCounterPlugin;
//# sourceMappingURL=plugin.js.map